@@ -25,7 +25,7 @@ fn bench_verify_account_changes_data(bencher: &mut Bencher) {
             &Rent::default(),
             &post,
             &mut ExecuteDetailsTimings::default(),
-            true
+            false,
         ),
         Ok(())
     );
@@ -38,7 +38,7 @@ fn bench_verify_account_changes_data(bencher: &mut Bencher) {
             &Rent::default(),
             &post,
             &mut ExecuteDetailsTimings::default(),
-            true,
+            false,
         )
         .unwrap();
     });
@@ -62,7 +62,7 @@ fn bench_verify_account_changes_data(bencher: &mut Bencher) {
             &Rent::default(),
             &post,
             &mut ExecuteDetailsTimings::default(),
-            true,
+            false,
         )
         .unwrap();
     });
