@@ -154,7 +154,7 @@ impl SnapshotRequestHandler {
                 // accounts that were included in the bank delta hash when the bank was frozen,
                 // and if we clean them here, the newly created snapshot's hash may not match
                 // the frozen hash.
-                snapshot_root_bank.clean_accounts(true);
+                snapshot_root_bank.clean_accounts(true, false);
                 clean_time.stop();
 
                 if accounts_db_caching_enabled {
@@ -263,11 +263,12 @@ impl AbsRequestHandler {
             })
     }
 
-    pub fn handle_pruned_banks(&self, bank: &Bank) -> usize {
+    /// `is_from_abs` is true if the caller is the AccountsBackgroundService
+    pub fn handle_pruned_banks(&self, bank: &Bank, is_from_abs: bool) -> usize {
         let mut count = 0;
         for pruned_slot in self.pruned_banks_receiver.try_iter() {
             count += 1;
-            bank.rc.accounts.purge_slot(pruned_slot);
+            bank.rc.accounts.purge_slot(pruned_slot, is_from_abs);
         }
 
         count
@@ -295,7 +296,7 @@ impl AccountsBackgroundService {
         let mut total_remove_slots_time = 0;
         let mut last_expiration_check_time = Instant::now();
         let t_background = Builder::new()
-            .name("solana-accounts-background".to_string())
+            .name("solana-bg-accounts".to_string())
             .spawn(move || loop {
                 if exit.load(Ordering::Relaxed) {
                     break;
@@ -372,7 +373,7 @@ impl AccountsBackgroundService {
                             // slots >= bank.slot()
                             bank.force_flush_accounts_cache();
                         }
-                        bank.clean_accounts(true);
+                        bank.clean_accounts(true, false);
                         last_cleaned_block_height = bank.block_height();
                     }
                 }
@@ -393,7 +394,7 @@ impl AccountsBackgroundService {
         total_remove_slots_time: &mut u64,
     ) {
         let mut remove_slots_time = Measure::start("remove_slots_time");
-        *removed_slots_count += request_handler.handle_pruned_banks(&bank);
+        *removed_slots_count += request_handler.handle_pruned_banks(&bank, true);
         remove_slots_time.stop();
         *total_remove_slots_time += remove_slots_time.as_us();
 
@@ -444,9 +445,11 @@ mod test {
         );
         assert!(bank0.get_account(&account_key).is_some());
         pruned_banks_sender.send(0).unwrap();
+
+        assert!(!bank0.rc.accounts.scan_slot(0, |_| Some(())).is_empty());
+
         AccountsBackgroundService::remove_dead_slots(&bank0, &request_handler, &mut 0, &mut 0);
 
-        // Slot should be removed
-        assert!(bank0.get_account(&account_key).is_none());
+        assert!(bank0.rc.accounts.scan_slot(0, |_| Some(())).is_empty());
     }
 }
