@@ -2,8 +2,9 @@ use {
     crate::{
         accounts::Accounts,
         accounts_db::{AccountStorageEntry, AccountsDb, AppendVecId, BankHashInfo},
-        accounts_index::{AccountSecondaryIndexes, Ancestors},
-        append_vec::AppendVec,
+        accounts_index::AccountSecondaryIndexes,
+        ancestors::Ancestors,
+        append_vec::{AppendVec, StoredMetaWriteVersion},
         bank::{Bank, BankFieldsToDeserialize, BankRc, Builtins},
         blockhash_queue::BlockhashQueue,
         epoch_stakes::EpochStakes,
@@ -64,7 +65,12 @@ pub(crate) enum SerdeStyle {
 const MAX_STREAM_SIZE: u64 = 32 * 1024 * 1024 * 1024;
 
 #[derive(Clone, Debug, Default, Deserialize, Serialize, AbiExample)]
-struct AccountsDbFields<T>(HashMap<Slot, Vec<T>>, u64, Slot, BankHashInfo);
+struct AccountsDbFields<T>(
+    HashMap<Slot, Vec<T>>,
+    StoredMetaWriteVersion,
+    Slot,
+    BankHashInfo,
+);
 
 trait TypeContext<'a> {
     type SerializableAccountStorageEntry: Serialize
@@ -129,6 +135,7 @@ pub(crate) fn bank_from_stream<R>(
     additional_builtins: Option<&Builtins>,
     account_indexes: AccountSecondaryIndexes,
     caching_enabled: bool,
+    limit_load_slot_count_from_snapshot: Option<usize>,
 ) -> std::result::Result<Bank, Error>
 where
     R: Read,
@@ -148,6 +155,7 @@ where
                 additional_builtins,
                 account_indexes,
                 caching_enabled,
+                limit_load_slot_count_from_snapshot,
             )?;
             Ok(bank)
         }};
@@ -237,6 +245,7 @@ fn reconstruct_bank_from_fields<E>(
     additional_builtins: Option<&Builtins>,
     account_indexes: AccountSecondaryIndexes,
     caching_enabled: bool,
+    limit_load_slot_count_from_snapshot: Option<usize>,
 ) -> Result<Bank, Error>
 where
     E: SerializableStorage,
@@ -248,8 +257,12 @@ where
         &genesis_config.cluster_type,
         account_indexes,
         caching_enabled,
+        limit_load_slot_count_from_snapshot,
     )?;
-    accounts_db.freeze_accounts(&bank_fields.ancestors, frozen_account_pubkeys);
+    accounts_db.freeze_accounts(
+        &Ancestors::from(&bank_fields.ancestors),
+        frozen_account_pubkeys,
+    );
 
     let bank_rc = BankRc::new(Accounts::new_empty(accounts_db), bank_fields.slot);
     let bank = Bank::new_from_fields(
@@ -270,6 +283,7 @@ fn reconstruct_accountsdb_from_fields<E>(
     cluster_type: &ClusterType,
     account_indexes: AccountSecondaryIndexes,
     caching_enabled: bool,
+    limit_load_slot_count_from_snapshot: Option<usize>,
 ) -> Result<AccountsDb, Error>
 where
     E: SerializableStorage,
@@ -362,6 +376,6 @@ where
     accounts_db
         .write_version
         .fetch_add(version, Ordering::Relaxed);
-    accounts_db.generate_index();
+    accounts_db.generate_index(limit_load_slot_count_from_snapshot);
     Ok(accounts_db)
 }
