@@ -10,7 +10,7 @@ use {
     bincode::serialize_into,
     rand::{thread_rng, Rng},
     solana_sdk::{
-        account::AccountSharedData,
+        account::{AccountSharedData, ReadableAccount},
         clock::Slot,
         genesis_config::{create_genesis_config, ClusterType},
         pubkey::Pubkey,
@@ -46,9 +46,9 @@ fn check_accounts(accounts: &Accounts, pubkeys: &[Pubkey], num: usize) {
     for _ in 1..num {
         let idx = thread_rng().gen_range(0, num - 1);
         let ancestors = vec![(0, 0)].into_iter().collect();
-        let account = accounts.load_slow(&ancestors, &pubkeys[idx]);
+        let account = accounts.load_without_fixed_root(&ancestors, &pubkeys[idx]);
         let account1 = Some((
-            AccountSharedData::new((idx + 1) as u64, 0, &AccountSharedData::default().owner),
+            AccountSharedData::new((idx + 1) as u64, 0, AccountSharedData::default().owner()),
             0,
         ));
         assert_eq!(account, account1);
@@ -73,6 +73,7 @@ where
         &ClusterType::Development,
         AccountSecondaryIndexes::default(),
         false,
+        None,
     )
 }
 
@@ -177,17 +178,17 @@ fn test_bank_serialize_style(serde_style: SerdeStyle) {
 
     // Create an account on a non-root fork
     let key1 = Keypair::new();
-    bank1.deposit(&key1.pubkey(), 5);
+    bank1.deposit(&key1.pubkey(), 5).unwrap();
 
     let bank2 = Bank::new_from_parent(&bank0, &Pubkey::default(), 2);
 
     // Test new account
     let key2 = Keypair::new();
-    bank2.deposit(&key2.pubkey(), 10);
+    bank2.deposit(&key2.pubkey(), 10).unwrap();
     assert_eq!(bank2.get_balance(&key2.pubkey()), 10);
 
     let key3 = Keypair::new();
-    bank2.deposit(&key3.pubkey(), 0);
+    bank2.deposit(&key3.pubkey(), 0).unwrap();
 
     bank2.freeze();
     bank2.squash();
@@ -226,6 +227,7 @@ fn test_bank_serialize_style(serde_style: SerdeStyle) {
         None,
         AccountSecondaryIndexes::default(),
         false,
+        None,
     )
     .unwrap();
     dbank.src = ref_sc;
@@ -286,7 +288,7 @@ fn test_bank_serialize_newer() {
 mod test_bank_serialize {
     use super::*;
 
-    // These some what long test harness is required to freeze the ABI of
+    // This some what long test harness is required to freeze the ABI of
     // Bank's serialization due to versioned nature
     #[frozen_abi(digest = "DuRGntVwLGNAv5KooafUSpxk67BPAx2yC7Z8A9c8wr2G")]
     #[derive(Serialize, AbiExample)]
