@@ -110,8 +110,8 @@ impl Stakes {
     }
 
     pub fn is_stake(account: &AccountSharedData) -> bool {
-        solana_vote_program::check_id(&account.owner)
-            || solana_stake_program::check_id(&account.owner)
+        solana_vote_program::check_id(account.owner())
+            || solana_stake_program::check_id(account.owner())
                 && account.data().len() >= std::mem::size_of::<StakeState>()
     }
 
@@ -121,16 +121,15 @@ impl Stakes {
         account: &AccountSharedData,
         fix_stake_deactivate: bool,
         check_vote_init: bool,
-        remove_delegation_on_inactive: bool,
     ) -> Option<ArcVoteAccount> {
-        if solana_vote_program::check_id(&account.owner) {
+        if solana_vote_program::check_id(account.owner()) {
             // unconditionally remove existing at first; there is no dependent calculated state for
             // votes, not like stakes (stake codepath maintains calculated stake value grouped by
             // delegated vote pubkey)
             let old = self.vote_accounts.remove(pubkey);
             // when account is removed (lamports == 0 or data uninitialized), don't read so that
             // given `pubkey` can be used for any owner in the future, while not affecting Stakes.
-            if account.lamports != 0
+            if account.lamports() != 0
                 && !(check_vote_init && VoteState::is_uninitialized_no_deser(&account.data()))
             {
                 let stake = old.as_ref().map_or_else(
@@ -149,7 +148,7 @@ impl Stakes {
                     .insert(*pubkey, (stake, ArcVoteAccount::from(account.clone())));
             }
             old.map(|(_, account)| account)
-        } else if solana_stake_program::check_id(&account.owner) {
+        } else if solana_stake_program::check_id(account.owner()) {
             //  old_stake is stake lamports and voter_pubkey from the pre-store() version
             let old_stake = self.stake_delegations.get(pubkey).map(|delegation| {
                 (
@@ -163,7 +162,7 @@ impl Stakes {
             let stake = delegation.map(|delegation| {
                 (
                     delegation.voter_pubkey,
-                    if account.lamports != 0 {
+                    if account.lamports() != 0 {
                         delegation.stake(
                             self.epoch,
                             Some(&self.stake_history),
@@ -188,13 +187,7 @@ impl Stakes {
                 }
             }
 
-            let remove_delegation = if remove_delegation_on_inactive {
-                delegation.is_none()
-            } else {
-                account.lamports == 0
-            };
-
-            if remove_delegation {
+            if account.lamports() == 0 {
                 // when account is removed (lamports == 0), remove it from Stakes as well
                 // so that given `pubkey` can be used for any owner in the future, while not
                 // affecting Stakes.
@@ -237,7 +230,7 @@ impl Stakes {
 #[cfg(test)]
 pub mod tests {
     use super::*;
-    use solana_sdk::{pubkey::Pubkey, rent::Rent};
+    use solana_sdk::{account::WritableAccount, pubkey::Pubkey, rent::Rent};
     use solana_stake_program::stake_state;
     use solana_vote_program::vote_state::{self, VoteState, VoteStateVersions};
 
@@ -313,8 +306,8 @@ pub mod tests {
             let ((vote_pubkey, vote_account), (stake_pubkey, mut stake_account)) =
                 create_staked_node_accounts(10);
 
-            stakes.store(&vote_pubkey, &vote_account, true, true, true);
-            stakes.store(&stake_pubkey, &stake_account, true, true, true);
+            stakes.store(&vote_pubkey, &vote_account, true, true);
+            stakes.store(&stake_pubkey, &stake_account, true, true);
             let stake = StakeState::stake_from(&stake_account).unwrap();
             {
                 let vote_accounts = stakes.vote_accounts();
@@ -325,8 +318,8 @@ pub mod tests {
                 );
             }
 
-            stake_account.lamports = 42;
-            stakes.store(&stake_pubkey, &stake_account, true, true, true);
+            stake_account.set_lamports(42);
+            stakes.store(&stake_pubkey, &stake_account, true, true);
             {
                 let vote_accounts = stakes.vote_accounts();
                 assert!(vote_accounts.get(&vote_pubkey).is_some());
@@ -338,7 +331,7 @@ pub mod tests {
 
             // activate more
             let (_stake_pubkey, mut stake_account) = create_stake_account(42, &vote_pubkey);
-            stakes.store(&stake_pubkey, &stake_account, true, true, true);
+            stakes.store(&stake_pubkey, &stake_account, true, true);
             let stake = StakeState::stake_from(&stake_account).unwrap();
             {
                 let vote_accounts = stakes.vote_accounts();
@@ -349,8 +342,8 @@ pub mod tests {
                 ); // now stake of 42 is activated
             }
 
-            stake_account.lamports = 0;
-            stakes.store(&stake_pubkey, &stake_account, true, true, true);
+            stake_account.set_lamports(0);
+            stakes.store(&stake_pubkey, &stake_account, true, true);
             {
                 let vote_accounts = stakes.vote_accounts();
                 assert!(vote_accounts.get(&vote_pubkey).is_some());
@@ -368,14 +361,14 @@ pub mod tests {
         let ((vote_pubkey, vote_account), (stake_pubkey, stake_account)) =
             create_staked_node_accounts(10);
 
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
-        stakes.store(&stake_pubkey, &stake_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
+        stakes.store(&stake_pubkey, &stake_account, true, true);
 
         let ((vote11_pubkey, vote11_account), (stake11_pubkey, stake11_account)) =
             create_staked_node_accounts(20);
 
-        stakes.store(&vote11_pubkey, &vote11_account, true, true, true);
-        stakes.store(&stake11_pubkey, &stake11_account, true, true, true);
+        stakes.store(&vote11_pubkey, &vote11_account, true, true);
+        stakes.store(&stake11_pubkey, &stake11_account, true, true);
 
         let vote11_node_pubkey = VoteState::from(&vote11_account).unwrap().node_pubkey;
 
@@ -392,8 +385,8 @@ pub mod tests {
         let ((vote_pubkey, mut vote_account), (stake_pubkey, stake_account)) =
             create_staked_node_accounts(10);
 
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
-        stakes.store(&stake_pubkey, &stake_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
+        stakes.store(&stake_pubkey, &stake_account, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -401,16 +394,16 @@ pub mod tests {
             assert_eq!(vote_accounts.get(&vote_pubkey).unwrap().0, 10);
         }
 
-        vote_account.lamports = 0;
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
+        vote_account.set_lamports(0);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
             assert!(vote_accounts.get(&vote_pubkey).is_none());
         }
 
-        vote_account.lamports = 1;
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
+        vote_account.set_lamports(1);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -420,8 +413,10 @@ pub mod tests {
 
         // Vote account too big
         let cache_data = vote_account.data().to_vec();
-        vote_account.data.push(0);
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
+        let mut pushed = vote_account.data().to_vec();
+        pushed.push(0);
+        vote_account.set_data(pushed);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -432,7 +427,7 @@ pub mod tests {
         let default_vote_state = VoteState::default();
         let versioned = VoteStateVersions::new_current(default_vote_state);
         VoteState::to(&versioned, &mut vote_account).unwrap();
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -440,7 +435,7 @@ pub mod tests {
         }
 
         vote_account.set_data(cache_data);
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -462,11 +457,11 @@ pub mod tests {
         let ((vote_pubkey2, vote_account2), (_stake_pubkey2, stake_account2)) =
             create_staked_node_accounts(10);
 
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
-        stakes.store(&vote_pubkey2, &vote_account2, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
+        stakes.store(&vote_pubkey2, &vote_account2, true, true);
 
         // delegates to vote_pubkey
-        stakes.store(&stake_pubkey, &stake_account, true, true, true);
+        stakes.store(&stake_pubkey, &stake_account, true, true);
 
         let stake = StakeState::stake_from(&stake_account).unwrap();
 
@@ -482,7 +477,7 @@ pub mod tests {
         }
 
         // delegates to vote_pubkey2
-        stakes.store(&stake_pubkey, &stake_account2, true, true, true);
+        stakes.store(&stake_pubkey, &stake_account2, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -507,11 +502,11 @@ pub mod tests {
 
         let (stake_pubkey2, stake_account2) = create_stake_account(10, &vote_pubkey);
 
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
 
         // delegates to vote_pubkey
-        stakes.store(&stake_pubkey, &stake_account, true, true, true);
-        stakes.store(&stake_pubkey2, &stake_account2, true, true, true);
+        stakes.store(&stake_pubkey, &stake_account, true, true);
+        stakes.store(&stake_pubkey2, &stake_account2, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -526,8 +521,8 @@ pub mod tests {
         let ((vote_pubkey, vote_account), (stake_pubkey, stake_account)) =
             create_staked_node_accounts(10);
 
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
-        stakes.store(&stake_pubkey, &stake_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
+        stakes.store(&stake_pubkey, &stake_account, true, true);
         let stake = StakeState::stake_from(&stake_account).unwrap();
 
         {
@@ -557,8 +552,8 @@ pub mod tests {
         let ((vote_pubkey, vote_account), (stake_pubkey, stake_account)) =
             create_staked_node_accounts(10);
 
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
-        stakes.store(&stake_pubkey, &stake_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
+        stakes.store(&stake_pubkey, &stake_account, true, true);
 
         {
             let vote_accounts = stakes.vote_accounts();
@@ -572,7 +567,6 @@ pub mod tests {
             &AccountSharedData::new(1, 0, &solana_stake_program::id()),
             true,
             true,
-            true,
         );
         {
             let vote_accounts = stakes.vote_accounts();
@@ -602,8 +596,8 @@ pub mod tests {
         let genesis_epoch = 0;
         let ((vote_pubkey, vote_account), (stake_pubkey, stake_account)) =
             create_warming_staked_node_accounts(10, genesis_epoch);
-        stakes.store(&vote_pubkey, &vote_account, true, true, true);
-        stakes.store(&stake_pubkey, &stake_account, true, true, true);
+        stakes.store(&vote_pubkey, &vote_account, true, true);
+        stakes.store(&stake_pubkey, &stake_account, true, true);
 
         assert_eq!(stakes.vote_balance_and_staked(), 11);
         assert_eq!(stakes.vote_balance_and_warmed_staked(), 1);
