@@ -1,4 +1,4 @@
-use crate::accounts_index::Ancestors;
+use crate::ancestors::Ancestors;
 
 use log::*;
 use rand::{thread_rng, Rng};
@@ -143,7 +143,7 @@ impl<T: Serialize + Clone> StatusCache<T> {
         if let Some(stored_forks) = keymap.get(key_slice) {
             let res = stored_forks
                 .iter()
-                .find(|(f, _)| ancestors.get(f).is_some() || self.roots.get(f).is_some())
+                .find(|(f, _)| ancestors.get(f) || self.roots.get(f).is_some())
                 .cloned();
             if res.is_some() {
                 return res;
@@ -305,11 +305,11 @@ mod tests {
         let blockhash = hash(Hash::default().as_ref());
         let status_cache = BankStatusCache::default();
         assert_eq!(
-            status_cache.get_status(&sig, &blockhash, &HashMap::new()),
+            status_cache.get_status(&sig, &blockhash, &Ancestors::default()),
             None
         );
         assert_eq!(
-            status_cache.get_status_any_blockhash(&sig, &HashMap::new()),
+            status_cache.get_status_any_blockhash(&sig, &Ancestors::default()),
             None
         );
     }
@@ -336,7 +336,7 @@ mod tests {
         let sig = Signature::default();
         let mut status_cache = BankStatusCache::default();
         let blockhash = hash(Hash::default().as_ref());
-        let ancestors = HashMap::new();
+        let ancestors = Ancestors::default();
         status_cache.insert(&blockhash, &sig, 1, ());
         assert_eq!(status_cache.get_status(&sig, &blockhash, &ancestors), None);
         assert_eq!(
@@ -350,7 +350,7 @@ mod tests {
         let sig = Signature::default();
         let mut status_cache = BankStatusCache::default();
         let blockhash = hash(Hash::default().as_ref());
-        let ancestors = HashMap::new();
+        let ancestors = Ancestors::default();
         status_cache.insert(&blockhash, &sig, 0, ());
         status_cache.add_root(0);
         assert_eq!(
@@ -380,7 +380,7 @@ mod tests {
         let sig = Signature::default();
         let mut status_cache = BankStatusCache::default();
         let blockhash = hash(Hash::default().as_ref());
-        let ancestors = HashMap::new();
+        let ancestors = Ancestors::default();
         status_cache.insert(&blockhash, &sig, 0, ());
         for i in 0..(MAX_CACHE_ENTRIES + 1) {
             status_cache.add_root(i as u64);
@@ -393,7 +393,7 @@ mod tests {
         let sig = Signature::default();
         let mut status_cache = BankStatusCache::default();
         let blockhash = hash(Hash::default().as_ref());
-        let ancestors = HashMap::new();
+        let ancestors = Ancestors::default();
         status_cache.insert(&blockhash, &sig, 0, ());
         status_cache.add_root(0);
         status_cache.clear();
@@ -405,7 +405,7 @@ mod tests {
         let sig = Signature::default();
         let mut status_cache = BankStatusCache::default();
         let blockhash = hash(Hash::default().as_ref());
-        let ancestors = HashMap::new();
+        let ancestors = Ancestors::default();
         status_cache.add_root(0);
         status_cache.clear();
         status_cache.insert(&blockhash, &sig, 0, ());
@@ -478,9 +478,9 @@ mod tests {
         status_cache.insert(&blockhash, &sig, 1, ());
         status_cache.insert(&blockhash2, &sig, 1, ());
 
-        let mut ancestors0 = HashMap::new();
+        let mut ancestors0 = Ancestors::default();
         ancestors0.insert(0, 0);
-        let mut ancestors1 = HashMap::new();
+        let mut ancestors1 = Ancestors::default();
         ancestors1.insert(1, 0);
 
         // Clear slot 0 related data
