@@ -1,10 +1,10 @@
 use log::*;
 use solana_sdk::{
-    account::{AccountSharedData, ReadableAccount},
+    account::{AccountSharedData, ReadableAccount, WritableAccount},
     account_utils::StateMut,
-    feature_set, ic_msg,
+    ic_msg,
     instruction::InstructionError,
-    keyed_account::{from_keyed_account, get_signers, next_keyed_account, KeyedAccount},
+    keyed_account::{from_keyed_account, get_signers, keyed_account_at_index, KeyedAccount},
     nonce,
     nonce_keyed_account::NonceKeyedAccount,
     process_instruction::InvokeContext,
@@ -35,7 +35,7 @@ impl Address {
     fn create(
         address: &Pubkey,
         with_seed: Option<(&Pubkey, &str, &Pubkey)>,
-        invoke_context: &mut dyn InvokeContext,
+        invoke_context: &dyn InvokeContext,
     ) -> Result<Self, InstructionError> {
         let base = if let Some((base, seed, owner)) = with_seed {
             let address_with_seed = Pubkey::create_with_seed(base, seed, owner)?;
@@ -66,7 +66,7 @@ fn allocate(
     address: &Address,
     space: u64,
     signers: &HashSet<Pubkey>,
-    invoke_context: &mut dyn InvokeContext,
+    invoke_context: &dyn InvokeContext,
 ) -> Result<(), InstructionError> {
     if !address.is_signer(signers) {
         ic_msg!(
@@ -79,7 +79,7 @@ fn allocate(
 
     // if it looks like the `to` account is already in use, bail
     //   (note that the id check is also enforced by message_processor)
-    if !account.data().is_empty() || !system_program::check_id(&account.owner) {
+    if !account.data().is_empty() || !system_program::check_id(&account.owner()) {
         ic_msg!(
             invoke_context,
             "Allocate: account {:?} already in use",
@@ -108,10 +108,10 @@ fn assign(
     address: &Address,
     owner: &Pubkey,
     signers: &HashSet<Pubkey>,
-    invoke_context: &mut dyn InvokeContext,
+    invoke_context: &dyn InvokeContext,
 ) -> Result<(), InstructionError> {
     // no work to do, just return
-    if account.owner == *owner {
+    if account.owner() == owner {
         return Ok(());
     }
 
@@ -126,7 +126,7 @@ fn assign(
         return Err(SystemError::InvalidProgramId.into());
     }
 
-    account.owner = *owner;
+    account.set_owner(*owner);
     Ok(())
 }
 
@@ -136,7 +136,7 @@ fn allocate_and_assign(
     space: u64,
     owner: &Pubkey,
     signers: &HashSet<Pubkey>,
-    invoke_context: &mut dyn InvokeContext,
+    invoke_context: &dyn InvokeContext,
 ) -> Result<(), InstructionError> {
     allocate(to, to_address, space, signers, invoke_context)?;
     assign(to, to_address, owner, signers, invoke_context)
@@ -150,12 +150,12 @@ fn create_account(
     space: u64,
     owner: &Pubkey,
     signers: &HashSet<Pubkey>,
-    invoke_context: &mut dyn InvokeContext,
+    invoke_context: &dyn InvokeContext,
 ) -> Result<(), InstructionError> {
     // if it looks like the `to` account is already in use, bail
     {
         let to = &mut to.try_account_ref_mut()?;
-        if to.lamports > 0 {
+        if to.lamports() > 0 {
             ic_msg!(
                 invoke_context,
                 "Create Account: account {:?} already in use",
@@ -173,7 +173,7 @@ fn transfer_verified(
     from: &KeyedAccount,
     to: &KeyedAccount,
     lamports: u64,
-    invoke_context: &mut dyn InvokeContext,
+    invoke_context: &dyn InvokeContext,
 ) -> Result<(), InstructionError> {
     if !from.data_is_empty()? {
         ic_msg!(invoke_context, "Transfer: `from` must not carry data");
@@ -189,8 +189,8 @@ fn transfer_verified(
         return Err(SystemError::ResultWithNegativeLamports.into());
     }
 
-    from.try_account_ref_mut()?.lamports -= lamports;
-    to.try_account_ref_mut()?.lamports += lamports;
+    from.try_account_ref_mut()?.checked_sub_lamports(lamports)?;
+    to.try_account_ref_mut()?.checked_add_lamports(lamports)?;
     Ok(())
 }
 
@@ -198,11 +198,9 @@ fn transfer(
     from: &KeyedAccount,
     to: &KeyedAccount,
     lamports: u64,
-    invoke_context: &mut dyn InvokeContext,
+    invoke_context: &dyn InvokeContext,
 ) -> Result<(), InstructionError> {
-    if !invoke_context.is_feature_active(&feature_set::system_transfer_zero_check::id())
-        && lamports == 0
-    {
+    if lamports == 0 {
         return Ok(());
     }
 
@@ -225,11 +223,9 @@ fn transfer_with_seed(
     from_owner: &Pubkey,
     to: &KeyedAccount,
     lamports: u64,
-    invoke_context: &mut dyn InvokeContext,
+    invoke_context: &dyn InvokeContext,
 ) -> Result<(), InstructionError> {
-    if !invoke_context.is_feature_active(&feature_set::system_transfer_zero_check::id())
-        && lamports == 0
-    {
+    if lamports == 0 {
         return Ok(());
     }
 
@@ -259,17 +255,16 @@ fn transfer_with_seed(
 
 pub fn process_instruction(
     _owner: &Pubkey,
-    keyed_accounts: &[KeyedAccount],
     instruction_data: &[u8],
     invoke_context: &mut dyn InvokeContext,
 ) -> Result<(), InstructionError> {
+    let keyed_accounts = invoke_context.get_keyed_accounts()?;
     let instruction = limited_deserialize(instruction_data)?;
 
     trace!("process_instruction: {:?}", instruction);
     trace!("keyed_accounts: {:?}", keyed_accounts);
 
     let signers = get_signers(keyed_accounts);
-    let keyed_accounts_iter = &mut keyed_accounts.iter();
 
     match instruction {
         SystemInstruction::CreateAccount {
@@ -277,8 +272,8 @@ pub fn process_instruction(
             space,
             owner,
         } => {
-            let from = next_keyed_account(keyed_accounts_iter)?;
-            let to = next_keyed_account(keyed_accounts_iter)?;
+            let from = keyed_account_at_index(keyed_accounts, 0)?;
+            let to = keyed_account_at_index(keyed_accounts, 1)?;
             let to_address = Address::create(to.unsigned_key(), None, invoke_context)?;
             create_account(
                 from,
@@ -298,8 +293,8 @@ pub fn process_instruction(
             space,
             owner,
         } => {
-            let from = next_keyed_account(keyed_accounts_iter)?;
-            let to = next_keyed_account(keyed_accounts_iter)?;
+            let from = keyed_account_at_index(keyed_accounts, 0)?;
+            let to = keyed_account_at_index(keyed_accounts, 1)?;
             let to_address = Address::create(
                 &to.unsigned_key(),
                 Some((&base, &seed, &owner)),
@@ -317,14 +312,14 @@ pub fn process_instruction(
             )
         }
         SystemInstruction::Assign { owner } => {
-            let keyed_account = next_keyed_account(keyed_accounts_iter)?;
+            let keyed_account = keyed_account_at_index(keyed_accounts, 0)?;
             let mut account = keyed_account.try_account_ref_mut()?;
             let address = Address::create(keyed_account.unsigned_key(), None, invoke_context)?;
             assign(&mut account, &address, &owner, &signers, invoke_context)
         }
         SystemInstruction::Transfer { lamports } => {
-            let from = next_keyed_account(keyed_accounts_iter)?;
-            let to = next_keyed_account(keyed_accounts_iter)?;
+            let from = keyed_account_at_index(keyed_accounts, 0)?;
+            let to = keyed_account_at_index(keyed_accounts, 1)?;
             transfer(from, to, lamports, invoke_context)
         }
         SystemInstruction::TransferWithSeed {
@@ -332,9 +327,9 @@ pub fn process_instruction(
             from_seed,
             from_owner,
         } => {
-            let from = next_keyed_account(keyed_accounts_iter)?;
-            let base = next_keyed_account(keyed_accounts_iter)?;
-            let to = next_keyed_account(keyed_accounts_iter)?;
+            let from = keyed_account_at_index(keyed_accounts, 0)?;
+            let base = keyed_account_at_index(keyed_accounts, 1)?;
+            let to = keyed_account_at_index(keyed_accounts, 2)?;
             transfer_with_seed(
                 from,
                 base,
@@ -346,40 +341,49 @@ pub fn process_instruction(
             )
         }
         SystemInstruction::AdvanceNonceAccount => {
-            let me = &mut next_keyed_account(keyed_accounts_iter)?;
+            let me = &mut keyed_account_at_index(keyed_accounts, 0)?;
             me.advance_nonce_account(
-                &from_keyed_account::<RecentBlockhashes>(next_keyed_account(keyed_accounts_iter)?)?,
+                &from_keyed_account::<RecentBlockhashes>(keyed_account_at_index(
+                    keyed_accounts,
+                    1,
+                )?)?,
                 &signers,
                 invoke_context,
             )
         }
         SystemInstruction::WithdrawNonceAccount(lamports) => {
-            let me = &mut next_keyed_account(keyed_accounts_iter)?;
-            let to = &mut next_keyed_account(keyed_accounts_iter)?;
+            let me = &mut keyed_account_at_index(keyed_accounts, 0)?;
+            let to = &mut keyed_account_at_index(keyed_accounts, 1)?;
             me.withdraw_nonce_account(
                 lamports,
                 to,
-                &from_keyed_account::<RecentBlockhashes>(next_keyed_account(keyed_accounts_iter)?)?,
-                &from_keyed_account::<Rent>(next_keyed_account(keyed_accounts_iter)?)?,
+                &from_keyed_account::<RecentBlockhashes>(keyed_account_at_index(
+                    keyed_accounts,
+                    2,
+                )?)?,
+                &from_keyed_account::<Rent>(keyed_account_at_index(keyed_accounts, 3)?)?,
                 &signers,
                 invoke_context,
             )
         }
         SystemInstruction::InitializeNonceAccount(authorized) => {
-            let me = &mut next_keyed_account(keyed_accounts_iter)?;
+            let me = &mut keyed_account_at_index(keyed_accounts, 0)?;
             me.initialize_nonce_account(
                 &authorized,
-                &from_keyed_account::<RecentBlockhashes>(next_keyed_account(keyed_accounts_iter)?)?,
-                &from_keyed_account::<Rent>(next_keyed_account(keyed_accounts_iter)?)?,
+                &from_keyed_account::<RecentBlockhashes>(keyed_account_at_index(
+                    keyed_accounts,
+                    1,
+                )?)?,
+                &from_keyed_account::<Rent>(keyed_account_at_index(keyed_accounts, 2)?)?,
                 invoke_context,
             )
         }
         SystemInstruction::AuthorizeNonceAccount(nonce_authority) => {
-            let me = &mut next_keyed_account(keyed_accounts_iter)?;
+            let me = &mut keyed_account_at_index(keyed_accounts, 0)?;
             me.authorize_nonce_account(&nonce_authority, &signers, invoke_context)
         }
         SystemInstruction::Allocate { space } => {
-            let keyed_account = next_keyed_account(keyed_accounts_iter)?;
+            let keyed_account = keyed_account_at_index(keyed_accounts, 0)?;
             let mut account = keyed_account.try_account_ref_mut()?;
             let address = Address::create(keyed_account.unsigned_key(), None, invoke_context)?;
             allocate(&mut account, &address, space, &signers, invoke_context)
@@ -390,7 +394,7 @@ pub fn process_instruction(
             space,
             owner,
         } => {
-            let keyed_account = next_keyed_account(keyed_accounts_iter)?;
+            let keyed_account = keyed_account_at_index(keyed_accounts, 0)?;
             let mut account = keyed_account.try_account_ref_mut()?;
             let address = Address::create(
                 keyed_account.unsigned_key(),
@@ -407,7 +411,7 @@ pub fn process_instruction(
             )
         }
         SystemInstruction::AssignWithSeed { base, seed, owner } => {
-            let keyed_account = next_keyed_account(keyed_accounts_iter)?;
+            let keyed_account = keyed_account_at_index(keyed_accounts, 0)?;
             let mut account = keyed_account.try_account_ref_mut()?;
             let address = Address::create(
                 keyed_account.unsigned_key(),
@@ -426,7 +430,7 @@ pub enum SystemAccountKind {
 }
 
 pub fn get_system_account_kind(account: &AccountSharedData) -> Option<SystemAccountKind> {
-    if system_program::check_id(&account.owner) {
+    if system_program::check_id(account.owner()) {
         if account.data().is_empty() {
             Some(SystemAccountKind::System)
         } else if account.data().len() == nonce::State::size() {
@@ -479,14 +483,13 @@ mod tests {
 
     fn process_instruction(
         owner: &Pubkey,
-        keyed_accounts: &[KeyedAccount],
+        keyed_accounts: Vec<KeyedAccount>,
         instruction_data: &[u8],
     ) -> Result<(), InstructionError> {
         super::process_instruction(
             owner,
-            keyed_accounts,
             instruction_data,
-            &mut MockInvokeContext::default(),
+            &mut MockInvokeContext::new(keyed_accounts),
         )
     }
 
@@ -519,7 +522,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(&from, true, &from_account),
                     KeyedAccount::new(&to, true, &to_account)
                 ],
@@ -532,9 +535,9 @@ mod tests {
             ),
             Ok(())
         );
-        assert_eq!(from_account.borrow().lamports, 50);
-        assert_eq!(to_account.borrow().lamports, 50);
-        assert_eq!(to_account.borrow().owner, new_owner);
+        assert_eq!(from_account.borrow().lamports(), 50);
+        assert_eq!(to_account.borrow().lamports(), 50);
+        assert_eq!(to_account.borrow().owner(), &new_owner);
         assert_eq!(to_account.borrow().data(), &[0, 0]);
     }
 
@@ -551,7 +554,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(&from, true, &from_account),
                     KeyedAccount::new(&to, false, &to_account)
                 ],
@@ -566,9 +569,9 @@ mod tests {
             ),
             Ok(())
         );
-        assert_eq!(from_account.borrow().lamports, 50);
-        assert_eq!(to_account.borrow().lamports, 50);
-        assert_eq!(to_account.borrow().owner, new_owner);
+        assert_eq!(from_account.borrow().lamports(), 50);
+        assert_eq!(to_account.borrow().lamports(), 50);
+        assert_eq!(to_account.borrow().owner(), &new_owner);
         assert_eq!(to_account.borrow().data(), &[0, 0]);
     }
 
@@ -587,7 +590,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(&from, true, &from_account),
                     KeyedAccount::new(&to, false, &to_account),
                     KeyedAccount::new(&base, true, &base_account)
@@ -603,9 +606,9 @@ mod tests {
             ),
             Ok(())
         );
-        assert_eq!(from_account.borrow().lamports, 50);
-        assert_eq!(to_account.borrow().lamports, 50);
-        assert_eq!(to_account.borrow().owner, new_owner);
+        assert_eq!(from_account.borrow().lamports(), 50);
+        assert_eq!(to_account.borrow().lamports(), 50);
+        assert_eq!(to_account.borrow().owner(), &new_owner);
         assert_eq!(to_account.borrow().data(), &[0, 0]);
     }
 
@@ -620,7 +623,7 @@ mod tests {
             Address::create(
                 &to,
                 Some((&from, seed, &owner)),
-                &mut MockInvokeContext::default(),
+                &MockInvokeContext::new(vec![]),
             ),
             Err(SystemError::AddressWithSeedMismatch.into())
         );
@@ -638,7 +641,7 @@ mod tests {
         let to_address = Address::create(
             &to,
             Some((&from, seed, &new_owner)),
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         )
         .unwrap();
 
@@ -651,11 +654,11 @@ mod tests {
                 2,
                 &new_owner,
                 &HashSet::new(),
-                &mut MockInvokeContext::default(),
+                &MockInvokeContext::new(vec![]),
             ),
             Err(InstructionError::MissingRequiredSignature)
         );
-        assert_eq!(from_account.borrow().lamports, 100);
+        assert_eq!(from_account.borrow().lamports(), 100);
         assert_eq!(*to_account.borrow(), AccountSharedData::default());
     }
 
@@ -663,29 +666,29 @@ mod tests {
     fn test_create_with_zero_lamports() {
         // create account with zero lamports transferred
         let new_owner = Pubkey::new(&[9; 32]);
-        let from = Pubkey::new_unique();
-        let from_account = AccountSharedData::new_ref(100, 0, &Pubkey::new_unique()); // not from system account
+        let from = solana_sdk::pubkey::new_rand();
+        let from_account = AccountSharedData::new_ref(100, 1, &solana_sdk::pubkey::new_rand()); // not from system account
 
-        let to = Pubkey::new_unique();
+        let to = solana_sdk::pubkey::new_rand();
         let to_account = AccountSharedData::new_ref(0, 0, &Pubkey::default());
 
         assert_eq!(
             create_account(
-                &KeyedAccount::new(&from, true, &from_account),
-                &KeyedAccount::new(&to, true, &to_account),
+                &KeyedAccount::new(&from, false, &from_account), // no signer
+                &KeyedAccount::new(&to, false, &to_account),
                 &to.into(),
                 0,
                 2,
                 &new_owner,
-                &[from, to].iter().cloned().collect::<HashSet<_>>(),
-                &mut MockInvokeContext::default(),
+                &[to].iter().cloned().collect::<HashSet<_>>(),
+                &MockInvokeContext::new(vec![]),
             ),
             Ok(())
         );
 
-        let from_lamports = from_account.borrow().lamports;
-        let to_lamports = to_account.borrow().lamports;
-        let to_owner = to_account.borrow().owner;
+        let from_lamports = from_account.borrow().lamports();
+        let to_lamports = to_account.borrow().lamports();
+        let to_owner = *to_account.borrow().owner();
         assert_eq!(from_lamports, 100);
         assert_eq!(to_lamports, 0);
         assert_eq!(to_owner, new_owner);
@@ -710,7 +713,7 @@ mod tests {
             2,
             &new_owner,
             &[from, to].iter().cloned().collect::<HashSet<_>>(),
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(SystemError::ResultWithNegativeLamports.into()));
     }
@@ -734,7 +737,7 @@ mod tests {
             MAX_PERMITTED_DATA_LENGTH + 1,
             &system_program::id(),
             &signers,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert!(result.is_err());
         assert_eq!(
@@ -751,10 +754,10 @@ mod tests {
             MAX_PERMITTED_DATA_LENGTH,
             &system_program::id(),
             &signers,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert!(result.is_ok());
-        assert_eq!(to_account.borrow().lamports, 50);
+        assert_eq!(to_account.borrow().lamports(), 50);
         assert_eq!(
             to_account.borrow().data().len() as u64,
             MAX_PERMITTED_DATA_LENGTH
@@ -784,11 +787,11 @@ mod tests {
             2,
             &new_owner,
             &signers,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(SystemError::AccountAlreadyInUse.into()));
 
-        let from_lamports = from_account.borrow().lamports;
+        let from_lamports = from_account.borrow().lamports();
         assert_eq!(from_lamports, 100);
         assert_eq!(owned_account, unchanged_account);
 
@@ -803,10 +806,10 @@ mod tests {
             2,
             &new_owner,
             &signers,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(SystemError::AccountAlreadyInUse.into()));
-        let from_lamports = from_account.borrow().lamports;
+        let from_lamports = from_account.borrow().lamports();
         assert_eq!(from_lamports, 100);
         assert_eq!(*owned_account.borrow(), unchanged_account);
 
@@ -821,7 +824,7 @@ mod tests {
             2,
             &new_owner,
             &signers,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(SystemError::AccountAlreadyInUse.into()));
         assert_eq!(from_lamports, 100);
@@ -849,7 +852,7 @@ mod tests {
             2,
             &new_owner,
             &[owned_key].iter().cloned().collect::<HashSet<_>>(),
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(InstructionError::MissingRequiredSignature));
 
@@ -863,23 +866,23 @@ mod tests {
             2,
             &new_owner,
             &[from].iter().cloned().collect::<HashSet<_>>(),
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(InstructionError::MissingRequiredSignature));
 
-        // Don't support unsigned creation with zero lamports (ephemeral account)
+        // support creation/assignment with zero lamports (ephemeral account)
         let owned_account = AccountSharedData::new_ref(0, 0, &Pubkey::default());
         let result = create_account(
             &KeyedAccount::new(&from, false, &from_account),
-            &KeyedAccount::new(&owned_key, true, &owned_account),
+            &KeyedAccount::new(&owned_key, false, &owned_account),
             &owned_address,
             0,
             2,
             &new_owner,
             &[owned_key].iter().cloned().collect::<HashSet<_>>(),
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
-        assert_eq!(result, Err(InstructionError::MissingRequiredSignature));
+        assert_eq!(result, Ok(()));
     }
 
     #[test]
@@ -903,7 +906,7 @@ mod tests {
             2,
             &sysvar::id(),
             &signers,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
 
         assert_eq!(result, Err(SystemError::InvalidProgramId.into()));
@@ -937,7 +940,7 @@ mod tests {
             2,
             &new_owner,
             &signers,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(SystemError::AccountAlreadyInUse.into()));
     }
@@ -971,7 +974,7 @@ mod tests {
                 0,
                 &solana_sdk::pubkey::new_rand(),
                 &signers,
-                &mut MockInvokeContext::default(),
+                &MockInvokeContext::new(vec![]),
             ),
             Err(InstructionError::InvalidArgument),
         );
@@ -990,7 +993,7 @@ mod tests {
                 &pubkey.into(),
                 &new_owner,
                 &HashSet::new(),
-                &mut MockInvokeContext::default(),
+                &MockInvokeContext::new(vec![]),
             ),
             Err(InstructionError::MissingRequiredSignature)
         );
@@ -1001,7 +1004,7 @@ mod tests {
                 &pubkey.into(),
                 &system_program::id(),
                 &HashSet::new(),
-                &mut MockInvokeContext::default(),
+                &MockInvokeContext::new(vec![]),
             ),
             Ok(())
         );
@@ -1010,7 +1013,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[KeyedAccount::new(&pubkey, true, &account)],
+                vec![KeyedAccount::new(&pubkey, true, &account)],
                 &bincode::serialize(&SystemInstruction::Assign { owner: new_owner }).unwrap()
             ),
             Ok(())
@@ -1030,7 +1033,7 @@ mod tests {
                 &from.into(),
                 &new_owner,
                 &[from].iter().cloned().collect::<HashSet<_>>(),
-                &mut MockInvokeContext::default(),
+                &MockInvokeContext::new(vec![]),
             ),
             Err(SystemError::InvalidProgramId.into())
         );
@@ -1043,7 +1046,7 @@ mod tests {
             owner: solana_sdk::pubkey::new_rand(),
         };
         let data = serialize(&instruction).unwrap();
-        let result = process_instruction(&system_program::id(), &[], &data);
+        let result = process_instruction(&system_program::id(), vec![], &data);
         assert_eq!(result, Err(InstructionError::NotEnoughAccountKeys));
 
         let from = solana_sdk::pubkey::new_rand();
@@ -1053,7 +1056,7 @@ mod tests {
         let data = serialize(&instruction).unwrap();
         let result = process_instruction(
             &system_program::id(),
-            &[KeyedAccount::new(&from, true, &from_account)],
+            vec![KeyedAccount::new(&from, true, &from_account)],
             &data,
         );
         assert_eq!(result, Err(InstructionError::NotEnoughAccountKeys));
@@ -1071,11 +1074,11 @@ mod tests {
             &from_keyed_account,
             &to_keyed_account,
             50,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         )
         .unwrap();
-        let from_lamports = from_keyed_account.account.borrow().lamports;
-        let to_lamports = to_keyed_account.account.borrow().lamports;
+        let from_lamports = from_keyed_account.account.borrow().lamports();
+        let to_lamports = to_keyed_account.account.borrow().lamports();
         assert_eq!(from_lamports, 50);
         assert_eq!(to_lamports, 51);
 
@@ -1085,37 +1088,24 @@ mod tests {
             &from_keyed_account,
             &to_keyed_account,
             100,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(SystemError::ResultWithNegativeLamports.into()));
-        assert_eq!(from_keyed_account.account.borrow().lamports, 50);
-        assert_eq!(to_keyed_account.account.borrow().lamports, 51);
-
-        // test signed transfer of zero
-        assert!(transfer(
-            &from_keyed_account,
-            &to_keyed_account,
-            0,
-            &mut MockInvokeContext::default(),
-        )
-        .is_ok(),);
-        assert_eq!(from_keyed_account.account.borrow().lamports, 50);
-        assert_eq!(to_keyed_account.account.borrow().lamports, 51);
+        assert_eq!(from_keyed_account.account.borrow().lamports(), 50);
+        assert_eq!(to_keyed_account.account.borrow().lamports(), 51);
 
         // test unsigned transfer of zero
         let from_keyed_account = KeyedAccount::new(&from, false, &from_account);
 
-        assert_eq!(
-            transfer(
-                &from_keyed_account,
-                &to_keyed_account,
-                0,
-                &mut MockInvokeContext::default(),
-            ),
-            Err(InstructionError::MissingRequiredSignature)
-        );
-        assert_eq!(from_keyed_account.account.borrow().lamports, 50);
-        assert_eq!(to_keyed_account.account.borrow().lamports, 51);
+        assert!(transfer(
+            &from_keyed_account,
+            &to_keyed_account,
+            0,
+            &MockInvokeContext::new(vec![]),
+        )
+        .is_ok(),);
+        assert_eq!(from_keyed_account.account.borrow().lamports(), 50);
+        assert_eq!(to_keyed_account.account.borrow().lamports(), 51);
     }
 
     #[test]
@@ -1138,11 +1128,11 @@ mod tests {
             &from_owner,
             &to_keyed_account,
             50,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         )
         .unwrap();
-        let from_lamports = from_keyed_account.account.borrow().lamports;
-        let to_lamports = to_keyed_account.account.borrow().lamports;
+        let from_lamports = from_keyed_account.account.borrow().lamports();
+        let to_lamports = to_keyed_account.account.borrow().lamports();
         assert_eq!(from_lamports, 50);
         assert_eq!(to_lamports, 51);
 
@@ -1155,11 +1145,11 @@ mod tests {
             &from_owner,
             &to_keyed_account,
             100,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         );
         assert_eq!(result, Err(SystemError::ResultWithNegativeLamports.into()));
-        assert_eq!(from_keyed_account.account.borrow().lamports, 50);
-        assert_eq!(to_keyed_account.account.borrow().lamports, 51);
+        assert_eq!(from_keyed_account.account.borrow().lamports(), 50);
+        assert_eq!(to_keyed_account.account.borrow().lamports(), 51);
 
         // test unsigned transfer of zero
         let from_keyed_account = KeyedAccount::new(&from, false, &from_account);
@@ -1170,11 +1160,11 @@ mod tests {
             &from_owner,
             &to_keyed_account,
             0,
-            &mut MockInvokeContext::default(),
+            &MockInvokeContext::new(vec![]),
         )
         .is_ok(),);
-        assert_eq!(from_keyed_account.account.borrow().lamports, 50);
-        assert_eq!(to_keyed_account.account.borrow().lamports, 51);
+        assert_eq!(from_keyed_account.account.borrow().lamports(), 50);
+        assert_eq!(to_keyed_account.account.borrow().lamports(), 51);
     }
 
     #[test]
@@ -1201,7 +1191,7 @@ mod tests {
                 &KeyedAccount::new(&from, true, &from_account),
                 &KeyedAccount::new(&to, false, &to_account),
                 50,
-                &mut MockInvokeContext::default(),
+                &MockInvokeContext::new(vec![]),
             ),
             Err(InstructionError::InvalidArgument),
         )
@@ -1307,7 +1297,7 @@ mod tests {
             bank.force_flush_accounts_cache();
             // do clean and assert that it actually did its job
             assert_eq!(3, bank.get_snapshot_storages().len());
-            bank.clean_accounts(false);
+            bank.clean_accounts(false, false);
             assert_eq!(2, bank.get_snapshot_storages().len());
         });
     }
@@ -1408,7 +1398,7 @@ mod tests {
                 .zip(accounts.iter())
                 .map(|(meta, account)| KeyedAccount::new(&meta.pubkey, meta.is_signer, account))
                 .collect();
-            process_instruction(&Pubkey::default(), &keyed_accounts, &instruction.data)
+            process_instruction(&Pubkey::default(), keyed_accounts, &instruction.data)
         }
     }
 
@@ -1428,7 +1418,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[],
+                vec![],
                 &serialize(&SystemInstruction::AdvanceNonceAccount).unwrap()
             ),
             Err(InstructionError::NotEnoughAccountKeys),
@@ -1440,11 +1430,11 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[KeyedAccount::new(
+                vec![KeyedAccount::new(
                     &Pubkey::default(),
                     true,
                     &create_default_account(),
-                ),],
+                )],
                 &serialize(&SystemInstruction::AdvanceNonceAccount).unwrap(),
             ),
             Err(InstructionError::NotEnoughAccountKeys),
@@ -1456,7 +1446,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(&Pubkey::default(), true, &create_default_account()),
                     KeyedAccount::new(
                         &sysvar::recent_blockhashes::id(),
@@ -1475,7 +1465,7 @@ mod tests {
         let nonce_acc = nonce_account::create_account(1_000_000);
         process_instruction(
             &Pubkey::default(),
-            &[
+            vec![
                 KeyedAccount::new(&Pubkey::default(), true, &nonce_acc),
                 KeyedAccount::new(
                     &sysvar::recent_blockhashes::id(),
@@ -1503,7 +1493,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(&Pubkey::default(), true, &nonce_acc,),
                     KeyedAccount::new(
                         &sysvar::recent_blockhashes::id(),
@@ -1535,7 +1525,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[],
+                vec![],
                 &serialize(&SystemInstruction::WithdrawNonceAccount(42)).unwrap(),
             ),
             Err(InstructionError::NotEnoughAccountKeys),
@@ -1547,11 +1537,11 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[KeyedAccount::new(
+                vec![KeyedAccount::new(
                     &Pubkey::default(),
                     true,
                     &create_default_account()
-                ),],
+                )],
                 &serialize(&SystemInstruction::WithdrawNonceAccount(42)).unwrap(),
             ),
             Err(InstructionError::NotEnoughAccountKeys),
@@ -1563,7 +1553,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(&Pubkey::default(), true, &create_default_account()),
                     KeyedAccount::new(&Pubkey::default(), false, &create_default_account()),
                     KeyedAccount::new(
@@ -1583,7 +1573,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(
                         &Pubkey::default(),
                         true,
@@ -1608,7 +1598,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(
                         &Pubkey::default(),
                         true,
@@ -1633,7 +1623,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[],
+                vec![],
                 &serialize(&SystemInstruction::InitializeNonceAccount(Pubkey::default())).unwrap(),
             ),
             Err(InstructionError::NotEnoughAccountKeys),
@@ -1645,11 +1635,11 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[KeyedAccount::new(
+                vec![KeyedAccount::new(
                     &Pubkey::default(),
                     true,
                     &nonce_account::create_account(1_000_000),
-                ),],
+                )],
                 &serialize(&SystemInstruction::InitializeNonceAccount(Pubkey::default())).unwrap(),
             ),
             Err(InstructionError::NotEnoughAccountKeys),
@@ -1661,7 +1651,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(
                         &Pubkey::default(),
                         true,
@@ -1684,7 +1674,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(
                         &Pubkey::default(),
                         true,
@@ -1708,7 +1698,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[
+                vec![
                     KeyedAccount::new(
                         &Pubkey::default(),
                         true,
@@ -1732,7 +1722,7 @@ mod tests {
         let nonce_acc = nonce_account::create_account(1_000_000);
         process_instruction(
             &Pubkey::default(),
-            &[
+            vec![
                 KeyedAccount::new(&Pubkey::default(), true, &nonce_acc),
                 KeyedAccount::new(
                     &sysvar::recent_blockhashes::id(),
@@ -1747,7 +1737,7 @@ mod tests {
         assert_eq!(
             process_instruction(
                 &Pubkey::default(),
-                &[KeyedAccount::new(&Pubkey::default(), true, &nonce_acc,),],
+                vec![KeyedAccount::new(&Pubkey::default(), true, &nonce_acc,),],
                 &serialize(&SystemInstruction::AuthorizeNonceAccount(Pubkey::default(),)).unwrap(),
             ),
             Ok(()),
